#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Add health_status and health_status_reason to cluster

Revision ID: cbbc65a86986
Revises: 9a1539f1cd2c
Create Date: 2018-05-15 22:24:49.527558

"""

# revision identifiers, used by Alembic.
revision = 'cbbc65a86986'
down_revision = '9a1539f1cd2c'

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('cluster', sa.Column('health_status',
                                       sa.String(20),
                                       nullable=True))
    op.add_column('cluster', sa.Column('health_status_reason',
                                       sa.Text,
                                       nullable=True))
    # ### end Alembic commands ###
